/*
 * Copyright (c) 2019 Stefan Strobel <stefan.strobel@shimatta.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

/**
 * @file    shellmatta_utils.h
 * @brief   util/helper functions of shellmatta
 * @author  Stefan Strobel <stefan.strobel@shimatta.net>
 */

/**
 * @addtogroup shellmatta_utils
 * @{
 */
#ifndef _SHELLMATTA_UTILS_H_
#define _SHELLMATTA_UTILS_H_

#include "shellmatta.h"
#include <stdint.h>


#define SHELLMATTA_MIN(a,b) (((a) > (b)) ?  (b) : (a))
#define SHELLMATTA_MAX(a,b) (((a) < (b)) ?  (b) : (a))
#define SHELLMATTA_PRINT_BUFFER(buffer,cnt,fct) \
    while((cnt) > sizeof((buffer)))             \
    {                                           \
        (cnt) -= sizeof((buffer));              \
        (fct)((buffer), sizeof((buffer)));      \
    }                                           \
    if((cnt) != 0u)                             \
    {                                           \
        (fct)((buffer), (cnt));                 \
    }

extern shellmatta_cmd_t helpCmd;

#define SHELLMATTA_MAGIC    0x5101E110u

#ifndef SHELLMATTA_OUTPUT_BUFFER_SIZE
#define SHELLMATTA_OUTPUT_BUFFER_SIZE   128u
#endif

void utils_writeEcho(   shellmatta_instance_t   *inst,
                        const char              *data,
                        uint32_t                length);
uint32_t utils_shellItoa(int32_t value, char *buffer, uint32_t base);
void utils_saveCursorPos(shellmatta_instance_t *inst);
void utils_restoreCursorPos(shellmatta_instance_t *inst);
void utils_eraseLine(shellmatta_instance_t *inst);
void utils_rewindCursor(shellmatta_instance_t *inst, uint32_t length);
void utils_forwardCursor(shellmatta_instance_t *inst, uint32_t length);
void utils_insertChars( shellmatta_instance_t   *inst,
                        char                    *data,
                        uint32_t                 length);
void utils_removeChars( shellmatta_instance_t   *inst,
                        uint32_t                 length,
                        bool                     backspace);
void utils_clearInput(shellmatta_instance_t *inst);
void utils_terminateInput(shellmatta_instance_t *inst);

#endif

/** @} */

